#!/usr/bin/env python
#-*- coding:utf-8 -*-
# author:zbt
# editor:kanwuqing
# datetime:2020-03-16 11:53
# software: PyCharm
import shutil
import logging
class SensitiveFilter:
    '''敏感词检测器

    Args:
        path: 敏感词库位置
    '''
    def __init__(self, path: str = r'legendwcf\dfa\ban.txt'):
        # 读取敏感词库内容
        self.path = path
        self.load()
    
    def load(self):
        with open(self.path, 'r', encoding = 'utf-8') as f:
            self.sensitiveWordList = f.read().split('\n')
        #得到敏感词字典
        self.sensitiveWordMap = self.initSensitiveWordMap(self.sensitiveWordList)
        logging.info('敏感词检测初始化完成')
    
    def add(self, word):
        if word not in self.sensitiveWordList:
            shutil.copy(self.path, f'{self.path}.bk1')
            self.sensitiveWordList.append(word)
            with open(self.path, 'w', encoding = 'utf-8') as f:
                f.write('\n'.join(self.sensitiveWordList))
            self.load()
    
    def delete(self, word):
        if word in self.sensitiveWordList:
            shutil.copy(self.path, f'{self.path}.bk1')
            self.sensitiveWordList.remove(word)
            with open(self.path, 'w', encoding = 'utf-8') as f:
                f.write('\n'.join(self.sensitiveWordList))
            self.load()
    
    #构建敏感词库
    def initSensitiveWordMap(self,sensitiveWordList):
        sensitiveWordMap = {}
        # 读取每一行，每一个word都是一个敏感词
        for word in sensitiveWordList:
            nowMap=sensitiveWordMap
            #遍历该敏感词的每一个特定字符
            for i in range(len(word)):
                keychar=word[i]
                wordMap=nowMap.get(keychar)
                if wordMap !=None:
                    #nowMap更新为下一层
                    nowMap=wordMap
                else:
                    #不存在则构建一个map,isEnd设置为0，因为不是最后一个
                    newNextMap={}
                    newNextMap["isEnd"]=0
                    nowMap[keychar]=newNextMap
                    nowMap=newNextMap
                #到这个词末尾字符
                if i==len(word)-1:
                    nowMap["isEnd"]=1
        #print(sensitiveWordMap)
        return sensitiveWordMap

    def checkSensitiveWord(self,txt,beginIndex=0):
        '''
        :param txt: 输入待检测的文本
        :param beginIndex: 输入文本开始的下标
        :return: 返回敏感词字符的长度
        '''
        nowMap=self.sensitiveWordMap
        sensitiveWordLen=0 #敏感词的长度
        containChar_sensitiveWordLen=0 #包括特殊字符敏感词的长度
        endFlag=False #结束标记位

        for i in range(beginIndex,len(txt)):
            char=txt[i]

            nowMap=nowMap.get(char)
            if nowMap != None:
                sensitiveWordLen+=1
                containChar_sensitiveWordLen+=1
                #结束位置为True
                if nowMap.get("isEnd")==1:
                    endFlag=True
            else:
                break
        if  endFlag==False:
            containChar_sensitiveWordLen=0
        #print(sensitiveWordLen)
        return containChar_sensitiveWordLen

    def getSensitiveWord(self,txt) -> list:
        '''返回所有敏感词
        '''
        cur_txt_sensitiveList=[]
        #注意，并不是一个个char查找的，找到敏感词会i增强敏感词的长度
        for i in range(len(txt)):
            length=self.checkSensitiveWord(txt,i)
            if length>0:
                word=txt[i:i+length]
                cur_txt_sensitiveList.append(word)
                i=i+length-1 #出了循环还要+1 i+length是没有检测到的，下次直接从i+length开始

        return cur_txt_sensitiveList

    def replaceSensitiveWord(self,txt: str,replaceChar='*') -> str:
        '''对字符串进行去敏

        Args:
            txt: 待去敏的文本
            replaceChar: 替换字符
        
        Returns:
            去敏后的字符串
        '''
        Lst=self.getSensitiveWord(txt)
        #print(Lst)
        for word in Lst:
            replaceStr=len(word)*replaceChar
            txt=txt.replace(word,replaceStr)

        return txt

if __name__ == "__main__":
    str="鸡你太美"
    Filter=SensitiveFilter()
    replaceStr=Filter.replaceSensitiveWord(str)
    print(replaceStr)
    Filter.add('你好')
    replaceStr=Filter.replaceSensitiveWord('你好')
    print(replaceStr)
    Filter.delete('你好')
    replaceStr=Filter.replaceSensitiveWord('你好')
    print(replaceStr)




import apscheduler.job
import apscheduler.schedulers
import apscheduler.schedulers.background
from apscheduler.triggers.cron import CronTrigger
import time
import apscheduler
from typing import Any, Callable
from datetime import datetime, timedelta, timezone

class LegendJob(apscheduler.schedulers.background.BackgroundScheduler):
    '''微信机器人定时任务管理
    '''
    def __init__(self, gconfig=..., **options):
        super().__init__(gconfig, **options)
    
    def one_time_job(self, func: Callable, microseconds: int = 0, seconds: int = 0, minutes: int = 0, hours: int = 0, days: int = 0, months: int = 0, years: int = 0, timedelta_: timedelta = None, datetime_: datetime | str = None,id: Any = None, name: Any = None, args: Any = None, kwargs: Any = None, **trigger_args: Any) -> apscheduler.job.Job | int:
        '''仅执行一次的任务

        time_>timedelta_>microseconds~years

        Args:
            timedelta_: 过多久执行, timedelta精确时间间隔
            time_: 执行时间, datetime精确时间或类似'YYYY-MM-DD HH:MM:SS'的字符串形式
        
        Returns:
            运行正常则返回Job类型对象, 表示添加的任务

            返回0代表任务与现在间隔太短或是过去时间
        '''
        if datetime_:
            return self.add_job(func, 'date', run_date=datetime_, name=name, id=id, args=args, kwargs=kwargs, **trigger_args)
        elif timedelta_:
            now = datetime.now()
            if timedelta_.total_seconds <= 3:
                return 0
            run = now + timedelta_
            return self.add_job(func, 'date', run_date=run, name=name, id=id, args=args, kwargs=kwargs, **trigger_args)
        else:
            now = datetime.now()
            add = timedelta(year=years, month=months, day=days, hour=hours, minute=minutes, second=seconds, microsecond=microseconds)
            if add.total_seconds <= 3:
                return 0
            run = now + add
            return self.add_job(func, 'date', run_date=run, name=name, id=id, args=args, kwargs=kwargs, **trigger_args)
    
    def interval_job(self, func: Callable, microseconds: int = 0, seconds: int = 0, minutes: int = 0, hours: int = 0, days: int = 0, weeks: int = 0, timedelta_: timedelta = None,jitter: int = 0, id: Any = None, name: Any = None, args: Any = None, kwargs: Any = None, **trigger_args: Any) -> apscheduler.job.Job:
        '''周期执行的任务

        timedelta_>microseconds~years

        Args:
            jitter: 振动参数, 在每次执行前随机添加随机浮动的秒数
            timedelta_: 每过多久执行, timedelta精确时间间隔
        
        Returns:
            Job类型对象, 表示添加的任务
        '''
        if timedelta_:
            return self.add_job(func, 'interval', seconds=timedelta_.total_seconds(), name=name, id=id, jitter=jitter, args=args, kwargs=kwargs, **trigger_args)
        else:
            timedelta_ = timedelta(weeks=weeks, days=days, hours=hours, minutes=minutes, seconds=seconds, microseconds=microseconds)
            return self.add_job(func, 'interval', seconds=timedelta_.total_seconds(), name=name, id=id, jitter=jitter, args=args, kwargs=kwargs, **trigger_args)
    
    def cron_job(self, func: Callable, year=None, month=None, day=None, week=None, day_of_week=None, hour=None, minute=None,second=None, start_date=None, end_date=None, timezone: timezone=timezone(timedelta(hours=8), name='Asia/Shanghai'), jitter=None, standard_cron: str = None, id: Any = None, name: Any = None, args: Any = None, kwargs: Any = None, **trigger_args: Any) -> apscheduler.job.Job:
        '''cron任务

        具体参数格式请参照官方CronTrigger类, ConTrigger优先
        Args:
            int|str year: 4-digit year
            int|str month: month (1-12)
            int|str day: day of month (1-31)
            int|str week: ISO week (1-53)
            int|str day_of_week: number or name of weekday (0-6 or mon,tue,wed,thu,fri,sat,sun)
            int|str hour: hour (0-23)
            int|str minute: minute (0-59)
            int|str second: second (0-59)
            datetime|str start_date: earliest possible date/time to trigger on (inclusive)
            datetime|str end_date: latest possible date/time to trigger on (inclusive)
            datetime.tzinfo|str timezone: time zone to use for the date/time calculations (defaults
                to scheduler timezone)
            jitter: delay the job execution by ``jitter`` seconds at most

            note: The first weekday is always **monday**.

            standard_cron: 可直接传入CronTrigger表达式
        Returns:
            Job类型对象, 表示添加的任务
        '''
        if standard_cron:
            return self.add_job(func, CronTrigger.from_crontab(standard_cron), id=id, name=name, args=args, kwargs=kwargs, **trigger_args)
        else:
            return self.add_job(func, 'cron', year=year, month=month, day=day, hour=hour, minute=minute, second=second, day_of_week=day_of_week, start_date=start_date, end_date=end_date, timezone=timezone, jitter=jitter, id=id, name=name, args=args, kwargs=kwargs, **trigger_args)

if __name__ == '__main__':
    
    def p(c):
        print(c)
    l = LegendJob({'apscheduler.executors.default': {
        'class': 'apscheduler.executors.pool:ThreadPoolExecutor',
        'max_workers': '20'
    },
    'apscheduler.executors.processpool': {
        'type': 'processpool',
        'max_workers': '5'
    }})
    l.cron_job(func=p, second='*', day_of_week='sat', args=['nihao'])
    l.start()
    while 1:
        time.sleep(1)


from wcferry import Wcf
import logging

def sendMsg(wcf: Wcf, msg: str, receiver: str, aters: list | str = None) -> int:
    """ wcferry发送文本消息的优化

    Args:
        wcf: wcferry客户端
        msg: 消息字符串
        receiver: 接收人wxid或者群id
        at_list: 要@的wxid, @所有人则传入'all'即可, 有多个则传入列表(列表里不能出现all)

    Returns:
        int: 0 为成功，其他失败
    """

    # msg中不需要加@的人
    # 若传入aters, 则默认receiver是群id
    ats = ""
    at_str = ""

    if aters:
        if aters == 'all':
            ats = '@所有人'
            at_str = 'notify@all'
        else:
            for ater in aters:
                # 根据 wxid 查找群昵称, 空格不能删
                ats += f'@{wcf.get_alias_in_chatroom(ater, receiver)} '
                at_str += f'{ater},'

    at_str = at_str[:-1] if at_str.endswith(',') else ...

    if ats == "":
        logging.info(f'send text to {receiver}')
        return wcf.send_text(msg, receiver)
    else:
        logging.info(f'send text to {receiver}, at {aters}')
        return wcf.send_text(f"{ats} {msg}", receiver, at_str)


import jieba
import jieba.posseg
import logging
import re

class LegendJieba:
    def __init__(self, userdict: str = None) -> None:
        '''Legend微信机器人的自然语言处理模块

        用户字典格式参照jieba模块的要求

        Args
            userdict: 用户字典(txt文件为宜)路径
        '''

        if userdict:
            jieba.load_userdict(userdict)
            logging.info('用户词典加载成功')
        
        logging.info('jieba分词初始化完成')

    def strtime(self, text):
        '''自然语言时间转成标准格式时间

        Args:
            text: 仅保证'(年)月日时(分)的处理结果

        Returns:
            两个str, 分别是转换后的年月日与时间(不包含日期则第一个返回值为None)
        '''
        text = text.replace("年", "-").replace("月", "-").replace("日", " ").replace("号", "").replace("/", "-").replace("点", ":").replace('时', ':').replace('分', '').strip()
        text = re.sub(r"\s+", " ", text)
        t = ""
        regex = r"(\d{1,2}:\d{1,2})"
        t =  re.search(regex, text)
        t = t.group(1) if t else None

        regex_list = [
            r"(\d{4}-\d{1,2}-\d{1,2})",
            r"(\d{1,2}-\d{1,2})"
        ]
        for regex in regex_list:
            d =  re.search(regex, text)
            if d:
                d = d.group(1)
                return d, t
        else:
            return None, t
    